/**
 * @file       dac_static_example.c
 * @version    1.0
 * @date       14 June 2019
 *
 * Copyright(C) Redpine Signals 2016
 * All rights reserved by Redpine Signals.
 *
 * @section License
 * This program should be used on your own responsibility.
 * Redpine Signals assumes no responsibility for any losses
 * incurred by customers or third parties arising from the use of this file.
 *
 * @brief This file contains example to describe DAC functionality in static mode.
 *
 * @section Description
 * Demonstrate the DAC functionality in static mode,In static mode apply the single
 * sample to DAC and generate the constant voltage on DAC output GPIO(ULP_GPIO4) 
 */

/* Includes ------------------------------------------------------------------*/
#include "rsi_chip.h"
#include "rsi_bod.h"
#include "aux_reference_volt_config.h"

/* User Configurable MACROs */
#define  SAMPLING_RATE      500000
float    vref               = 2.8; 
int16_t input_sample_data[1] = {0x33A};

/**
 * @brief  Event call back function..
 * @retval None
 */
void dac_callback(uint8_t event)
{
	/* Call back function never call in DAC static mode */
}
/**
 * @brief  Main program.
 * @param  None
 * @retval None
 */ 
int main(void)
{
	/* At this stage the MICROCONTROLLER clock setting is already configured,
	 * this is done through SystemInit() function which is called from startup
	 * file (startup_rs1xxxx.s) before to branch to application main.
	 * To reconfigure the default setting of SystemInit() function, refer to
	 * system_rs1xxxx.c file
	 */
	volatile uint32_t achived_sampling_rate = 0;
	float battery_status;
	float  max_ip_volt_scdc = 2.4;

	SystemCoreClockUpdate();

	/* Read the input supply of chip */
	battery_status = RSI_BOD_SoftTriggerGetBatteryStatus();

	/*If input to chip less than 2.4V then switch input voltage supply from SCDC to HPLDO */ 	
	if(battery_status < max_ip_volt_scdc)
	{
		RSI_IPMU_ProgramConfigData(hp_ldo_voltsel);		
	}

	/* Initialize DAC */ 	
	achived_sampling_rate = DAC_Init(STATIC_MODE_EN,SAMPLING_RATE,dac_callback);

	/* Configure reference voltage for analog peripheral ,here till 2.8V generate by using 
	   AUX_LDO so more than 2.8V enable LDO bypass mode */
	RSI_AUX_RefVoltageConfig(vref,battery_status);	

	DAC_WriteData(STATIC_MODE_EN,input_sample_data,1);	

	DAC_Start(STATIC_MODE_EN);

	while(1);		
}	
