#include "Flash_Intf.h"

uint32_t  RSI_FLASH_UnInitialize(void)
{
		return 0;
}

void qspi_gpio_revert_m4(void)
{
  uint32_t gpio_pin_no = 0;
  uint32_t gpio_mode = 15;
  uint32_t i;

  //! Enable QSPI  clock and hclock
  CLK_ENABLE_SET_2_REG = ((1 << 12)|(1 << 11));
  RSI_EGPIO_PadSelectionDisable(0);
  for(i = 0; i < 6; i++) {
     RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, (gpio_pin_no + i), gpio_mode);
   }
 }

uint32_t RSI_FLASH_Initialize(void)
{
	/*This macro needs to be enabled when flash based retention sleep is needed*/
	/*Init the QSPI configurations structure */
	spi_config_t spi_configs_init; 
	*(uint32_t *)(0x22000004) = 0x1; //put TA in reset
	qspi_gpio_revert_m4();
	GetQspiConfig(&spi_configs_init);

#ifdef EXTERNAL_FLASH
	RSI_EGPIO_PadSelectionEnable(PadSelectionEnable_1);
	RSI_EGPIO_PadSelectionEnable(PadSelectionEnable_2);
#else
	RSI_EGPIO_PadSelectionEnable(PadSelectionEnable);
#endif
	/*Receive enable for QSPI GPIO*/
	RSI_EGPIO_PadReceiverEnable(M4SS_QSPI_CLK);
	RSI_EGPIO_PadReceiverEnable(M4SS_QSPI_CSN0);
	RSI_EGPIO_PadReceiverEnable(M4SS_QSPI_D0);
	RSI_EGPIO_PadReceiverEnable(M4SS_QSPI_D1);
	RSI_EGPIO_PadReceiverEnable(M4SS_QSPI_D2);
	RSI_EGPIO_PadReceiverEnable(M4SS_QSPI_D3);

	/*Set GPIO pin MUX for QSPI*/
	RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, M4SS_QSPI_CLK, QSPI_MODE);
	RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, M4SS_QSPI_CSN0 , QSPI_MODE);
	RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, M4SS_QSPI_D0 , QSPI_MODE);
	RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, M4SS_QSPI_D1 , QSPI_MODE);
	RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, M4SS_QSPI_D2 , QSPI_MODE);
	RSI_EGPIO_SetPinMux(QSPI_PINS_PORT, M4SS_QSPI_D3 , QSPI_MODE); 

	/*Initialize QSPI*/
	ROMAPI_QSPI_API->RSI_QSPI_SpiInit((qspi_reg_t *)(0x12000000),&spi_configs_init,1,0,0);
	
	return 0;
}


uint32_t RSI_FLASH_EraseSector(uint32_t sector_address)
{
	spi_config_t spi_configs_erase; 
	GetQspiConfig(&spi_configs_erase);
	ROMAPI_QSPI_API->RSI_QSPI_SpiErase((qspi_reg_t *)(0x12000000),&spi_configs_erase,SECTOR_ERASE, sector_address, 1, 0);
	return 0;   
}


uint32_t RSI_FLASH_Erasechip()
{
	volatile int sector_off_set_addr = 0;
	
	spi_config_t spi_configs_erase_chip; 
	GetQspiConfig(&spi_configs_erase_chip);

  /*Note : This function will not erase the MBR content of the flash but erases the rest of the flash from 72K*/	
	//! Initial offset of the flash to be erased.
	sector_off_set_addr = 0x12000;

	do{
	  ROMAPI_QSPI_API->RSI_QSPI_SpiErase((qspi_reg_t *)(0x12000000),&spi_configs_erase_chip,SECTOR_ERASE, sector_off_set_addr, 1, 0);
		//! Increment the sector address by 4K
		sector_off_set_addr = (sector_off_set_addr + (4*1024));
		//! This loop will be terminated once it reaches to the 1Mb range of memory
#ifdef RS14100_1MB		
	//! Compare with 1Mb flash size 	
	}while(sector_off_set_addr < 0x100000);
#else
	//! Compare with 4Mb flash size 	
	}while(sector_off_set_addr < 0x400000);
#endif //RS14100_1MB		 	
	return 0;
}

uint32_t checksum_addition(uint8_t *buf, uint32_t size, uint32_t prev_sum)
{
	uint32_t sum = prev_sum;
	uint32_t cnt;
	uint32_t cnt_limit;
	uint32_t dword;

	if (size == 0)
	{
 		return sum;
	}

	cnt_limit = (size & (~0x3));
	/* Accumulate checksum */
	for (cnt = 0; cnt < cnt_limit; cnt += 4)
	{
		dword = *(uint32_t *) &buf[cnt];
		sum += dword;
		if(sum < dword)
		{
			/* In addition operation, if result is lesser than any one of the operand
			 * it means carry is generated. 
			 * Incrementing the sum to get ones compliment addition */

			sum++;
		}
	}
	
	/* Handle non dword-sized case */
  if(size & 0x3) {
		dword = 0xffffffff;
		dword = ~(dword << (8 * (size & 0x3)));
		/* Keeping only valid bytes and making upper bytes zeroes. */
		dword = (*(uint32_t *) &buf[cnt]) & dword;
		sum += dword;
		if(sum < dword)
		{
			sum++;
		}
	}

	return ~sum;
}


uint32_t RSI_FLASH_ProgramPage(uint32_t address, unsigned char *data, uint32_t length)
{
	spi_config_t spi_configs_program; 
	uint32_t check_sum = 0; 
	GetQspiConfig(&spi_configs_program);
	if(address == 0x12000)
	{
		check_sum  = checksum_addition(data , 236 , 1);
		memcpy(&data[236] ,(uint8_t*)&check_sum,4);
	}
 	ROMAPI_QSPI_API->RSI_QSPI_SpiWrite((qspi_reg_t *)(0x12000000),&spi_configs_program,0x2 , address,(uint8_t *) data, length, 256, _1BYTE, 1,0,0,0,0,0);
	return 0;
}


uint32_t RSI_FLASH_Verify(uint32_t address, unsigned char *data, uint32_t length)
{
	uint32_t check_sum = 0, i; 
	uint8_t *buffer = (uint8_t *)address;
	
	if((address & 0x00FFFFFF)== 0x12000)	{
		check_sum  = checksum_addition(data , 236 , 1);
		memcpy(&data[236] ,(uint8_t*)&check_sum,4);	
		for(i=0;i<length;i++)
			if(buffer[i] != data[i])
				break;
	} else {
		for(i=0;i<length;i++)
			if(buffer[i] != data[i])
				break;
	}
	return (address+i);
}

uint32_t RSI_FLASH_Read(uint32_t address, unsigned char *data, uint32_t length, uint8_t auto_mode)
{
	return 0;
}

void GetQspiConfig(spi_config_t *spi_config)
{
	memset(spi_config, 0, sizeof(spi_config_t));
	spi_config->spi_config_1.inst_mode         = SINGLE_MODE;
	spi_config->spi_config_1.addr_mode         = SINGLE_MODE;
	spi_config->spi_config_1.data_mode         = SINGLE_MODE;
	spi_config->spi_config_1.dummy_mode        = SINGLE_MODE;
	spi_config->spi_config_1.extra_byte_mode   = SINGLE_MODE;
	spi_config->spi_config_1.prefetch_en       = DIS_PREFETCH;
	spi_config->spi_config_1.dummy_W_or_R      = DUMMY_READS;
	spi_config->spi_config_1.extra_byte_en     = 0;
	spi_config->spi_config_1.d3d2_data         = 3;
	spi_config->spi_config_1.continuous        = CONTINUOUS;
	spi_config->spi_config_1.read_cmd          = READ;
	spi_config->spi_config_1.flash_type        = SST_SPI_FLASH;
	spi_config->spi_config_1.no_of_dummy_bytes = 0;


	spi_config->spi_config_2.auto_mode         = EN_AUTO_MODE;
	spi_config->spi_config_2.cs_no             = CHIP_ZERO;
	spi_config->spi_config_2.neg_edge_sampling = NEG_EDGE_SAMPLING;
	spi_config->spi_config_2.qspi_clk_en       = QSPI_FULL_TIME_CLK;
	spi_config->spi_config_2.protection        = DNT_REM_WR_PROT;
	spi_config->spi_config_2.dma_mode          = NO_DMA;
	spi_config->spi_config_2.swap_en           = SWAP;
	spi_config->spi_config_2.full_duplex       = IGNORE_FULL_DUPLEX;
	spi_config->spi_config_2.wrap_len_in_bytes = NO_WRAP;
	spi_config->spi_config_2.addr_width_valid  = 0;
	spi_config->spi_config_2.addr_width        = _24BIT_ADDR;
	spi_config->spi_config_2.pinset_valid      = 0;
	spi_config->spi_config_2.flash_pinset      = GPIO_58_TO_63;
	spi_config->spi_config_2.dummy_cycles_for_controller = 0;

	spi_config->spi_config_3.xip_mode                = 0;
	spi_config->spi_config_3._16bit_cmd_valid        = 0;
	spi_config->spi_config_3._16bit_rd_cmd_msb       = 0;
	spi_config->spi_config_3.ddr_mode_en             = 0;
	spi_config->spi_config_3.wr_cmd                  = 0x2;
	spi_config->spi_config_3.wr_inst_mode            = SINGLE_MODE;
	spi_config->spi_config_3.wr_addr_mode            = SINGLE_MODE;
	spi_config->spi_config_3.wr_data_mode            = SINGLE_MODE;
	spi_config->spi_config_3.dummys_4_jump           = 1;

	spi_config->spi_config_4._16bit_wr_cmd_msb       = 0;
	spi_config->spi_config_4.qspi_ddr_clk_en         = 0;
	spi_config->spi_config_4.qspi_manual_ddr_phasse  = 0;
	spi_config->spi_config_4.ddr_data_mode           = 0;
	spi_config->spi_config_4.ddr_addr_mode           = 0;
	spi_config->spi_config_4.ddr_inst_mode           = 0;
	spi_config->spi_config_4.ddr_dummy_mode          = 0;
	spi_config->spi_config_4.ddr_extra_byte          = 0;
	spi_config->spi_config_4.dual_flash_mode         = 0;
	spi_config->spi_config_4.secondary_csn           = 1;
	spi_config->spi_config_4.polarity_mode           = 0;
	spi_config->spi_config_4.valid_prot_bits         = 4;
	spi_config->spi_config_4.no_of_ms_dummy_bytes    = 0;
	spi_config->spi_config_4.ddr_dll_en              = 0;
	spi_config->spi_config_4.continue_fetch_en       = 0;


	spi_config->spi_config_5.block_erase_cmd           = BLOCK_ERASE;
	spi_config->spi_config_5.busy_bit_pos              = 0;
	spi_config->spi_config_5.d7_d4_data                = 0xf;
	spi_config->spi_config_5.dummy_bytes_for_rdsr      = 0x0;
	spi_config->spi_config_5.reset_type                = 0x0;

	spi_config->spi_config_6.chip_erase_cmd        = CHIP_ERASE;
	spi_config->spi_config_6.sector_erase_cmd      = SECTOR_ERASE;

	spi_config->spi_config_7.status_reg_write_cmd  = 0x1;
	spi_config->spi_config_7.status_reg_read_cmd   = 0x5;
}
